﻿using System;
using System.Data;
using System.Xml;
using System.Xml.Linq;
using System.Linq;
using System.Configuration;

using CMS.SynchronizationEngine;
using CMS.Synchronization;
using CMS.SettingsProvider;
using CMS.DocumentEngine;
using CMS.DataEngine;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.GlobalHelper;
using CMS.WorkflowEngine;
using CMS.Ecommerce;
using CMS.IO;
using CMS.EventManager;
using CMS.EventLog;

/// <summary>
/// Sample integration connector for integration between two instances of CMS
/// </summary>
public class MyIntegrationConnector : BaseIntegrationConnector
{
    #region "Initialization (subscribing)"

    /// <summary>
    /// Initializes connector. Subscribes to desired objects.
    /// </summary>
    public override void Init()
    {
        ConnectorName = "MyIntegrationConnector";
        string siteName = "MySite";

        // Subscribe to 'CREATE' task type of EventAttendeeInfo
        ObjectIntegrationSubscription objCreateSub = new ObjectIntegrationSubscription(
            ConnectorName,
            TaskProcessTypeEnum.AsyncSimpleSnapshot,
            TaskTypeEnum.CreateObject,
            siteName,
            PredefinedObjectType.EVENTATTENDEE, 
            null
        );
        SubscribeTo(objCreateSub);
        
        // NOTE: "Subscriptions" are classes inheriting from AbstractIntegrationSubscription
        // - BaseIntegrationSubscription - e.g. AbstractIntegrationSubscription.GLOBAL_OBJECTS to subscribe only to global objects
	    // - ObjectIntegrationSubscription
	    // - DocumentIntegrationSubscription
        // - CUSTOM ?

        // Subscribe to 'UPDATE' task type of EventAttendeeInfo
        ObjectIntegrationSubscription objUpdateSub = new ObjectIntegrationSubscription(
            ConnectorName,
            TaskProcessTypeEnum.AsyncSnapshot,
            TaskTypeEnum.UpdateObject,
            siteName,
            PredefinedObjectType.EVENTATTENDEE,
            null
        );
        SubscribeTo(objUpdateSub);
        
        // Subscribe to 'DELETE' Synch! (SnapShot) task type of EventAttendeeInfo
        ObjectIntegrationSubscription objDeleteSub = new ObjectIntegrationSubscription(
            ConnectorName,
            TaskProcessTypeEnum.SyncSnapshot,
            TaskTypeEnum.DeleteObject,
            siteName,
            PredefinedObjectType.EVENTATTENDEE,
            null
        );
        SubscribeTo(objDeleteSub);

        // NOTE: 'DeleteObject' TaskType CAN NOT BE RUN Asynchronously!
    }

    #endregion


    #region "Asynchronous & Synchronous processing (Outbound)"

    /// <summary>
    /// Processes given object according to task type. All identifiers already match external database.
    /// It is expected that you use TranslateColumnsToExternal method before you process the task.
    /// The TranslateColumnsToExternal needs GetExternalObjectID and GetExternalDocumentID to be implemented.
    /// It traverses the given object and tries to translate foreign keys to match external (your) application.
    /// </summary>
    /// <param name="infoObj">Info object to process</param>
    /// <param name="translations">Translation helper object containing translations for given object</param>
    /// <param name="taskType">Type of task</param>
    /// <param name="dataType">Type of data</param>
    /// <param name="siteName">Name of site</param>
    /// <param name="errorMessage">Possible error message</param>
    /// <returns>Result of processing</returns>
    public override IntegrationProcessResultEnum ProcessInternalTaskAsync(GeneralizedInfo infoObj, TranslationHelper translations, TaskTypeEnum taskType, TaskDataTypeEnum dataType, string siteName, out string errorMessage)
    {
        switch (infoObj.ObjectType)
        {
            case PredefinedObjectType.EVENTATTENDEE:

                // Translate external columns
                this.TranslateColumnsToExternal(infoObj, translations, false);

                // Process task
                if (MyIntegrationConnectorHelper.ProcessObjectExternal(infoObj, taskType, out errorMessage))
                {
                    return IntegrationProcessResultEnum.OK;
                }
                else
                {
                    return IntegrationProcessResultEnum.Error;
                }

            default:

                errorMessage = "ERROR: Object type NOT DEFINED.";
                return IntegrationProcessResultEnum.ErrorAndSkip;
        }
    }


    /// <summary>
    /// Processes given object according to task type. All identifiers already match external database.
    /// </summary>
    /// <param name="infoObj">Info object to process</param>
    /// <param name="taskType">Type of task</param>
    /// <param name="siteName">Name of site</param>
    /// <param name="errorMessage">Possible error message</param>
    /// <returns>Result of processing</returns>
    public override IntegrationProcessResultEnum ProcessInternalTaskSync(GeneralizedInfo infoObj, TaskTypeEnum taskType, string siteName, out string errorMessage)
    {
        switch (infoObj.ObjectType)
        {
            case PredefinedObjectType.EVENTATTENDEE:

                // Performs custom translation, TranslationHelper is missing in sync processing!
                int nodeID = ValidationHelper.GetInteger(infoObj.GetValue("AttendeeEventNodeID"), 0);
                
                TreeProvider tp  = new TreeProvider(CMSContext.CurrentUser);
                TreeNode node = tp.SelectSingleNode(nodeID, "en-us");

                if (node != null)
                {
                    // My custom object column translation
                    infoObj.SetValue("AttendeeEventNodeID", MyIntegrationConnectorHelper.GetEventID(node.NodeGUID));

                    // Process task
                    if (MyIntegrationConnectorHelper.ProcessObjectExternal(infoObj, taskType, out errorMessage))
                    {
                        return IntegrationProcessResultEnum.OK;
                    }
                    else
                    {
                        return IntegrationProcessResultEnum.Error;
                    }
                }
                else
                {
                    errorMessage = "ERROR: Node does NOT EXISTS.";
                    return IntegrationProcessResultEnum.ErrorAndSkip;
                }

            default:

                errorMessage = "ERROR: Object type NOT DEFINED.";
                return IntegrationProcessResultEnum.ErrorAndSkip;
        }
    }

    #endregion


    #region "Retrieving identifiers (Outbound)"

    public override int GetExternalDocumentID(Guid nodeGuid, string cultureCode, string siteName, bool returnDocumentId)
    {
        return MyIntegrationConnectorHelper.GetEventID(nodeGuid);
    }
    
    #endregion


    #region "Retrieving translation parameters (Inbound)"

    /// <summary>
    /// Gets code name, site name, parent ID and group ID required for identifier translation.
    /// </summary>
    /// <param name="id">Identifier of object</param>
    /// <param name="objectType">Type of object</param>
    /// <param name="codeName">Returns code name of object</param>
    /// <param name="siteName">Returns site name of object</param>
    /// <param name="parentId">Returns identifier of parent object</param>
    /// <param name="groupId">Returns identifier of object community group</param>
    public override void GetInternalDocumentParams(int id, string className, out Guid nodeGuid, out string cultureCode, out string siteName)
    {
        // Based on given identifier and object type you should be able to identify object in your (external) application and return its parameters 
        // (marked as 'out')

        nodeGuid = MyIntegrationConnectorHelper.GetEventGUID(id);
        cultureCode = "";
        siteName = "MySite";
    }
    
    #endregion


    #region "Object transformation (Inbound)"

    /// <summary>
    /// Transforms given external object to internal (to TreeNode or GeneralizedInfo).
    /// </summary>
    /// <param name="obj">Object or document to transform</param>
    /// <param name="taskType">Type of task</param>
    /// <param name="dataType">Type of input data</param>
    /// <param name="siteName">Name of site</param>
    public override ICMSObject PrepareInternalObject(object obj, TaskTypeEnum taskType, TaskDataTypeEnum dataType, string siteName)
    {
        // This method is called withing IntegrationHelper.ProcessExternalTask and it provides you with space where you can easily transform
        // external object (possibly some kind of data container) to TreeNode (document) or GeneralizedInfo (object - eg. UserInfo)
        BaseInfo info = null;

        try
        {
            // Convert object to custom data object
            CustomData cd = new CustomData();
            cd.LoadData(ValidationHelper.GetString(obj, ""));

            // Get info by class name
            info = CMSObjectHelper.GetReadOnlyObject("cms.eventattendee");
            if (info != null)
            {
                string eventUserName = ValidationHelper.GetString(cd["EventUserName"], "");

                if (!string.IsNullOrEmpty(eventUserName) && eventUserName.Split(' ').Length >= 2)
                {
                    info.SetValue("AttendeeFirstName", eventUserName.Split(' ')[0]);
                    info.SetValue("AttendeeLastName", eventUserName.Split(' ')[1]);
                }
                else
                {
                    info.SetValue("AttendeeFirstName", eventUserName);
                    info.SetValue("AttendeeLastName", eventUserName);
                }
                
                info.SetValue("AttendeeEmail", ValidationHelper.GetString(cd["EventUserEmail"], ""));
                info.SetValue("AttendeeEventNodeID", ValidationHelper.GetInteger(cd["EventuserEventID"], 0));
                info.SetValue("AttendeeGUID", ValidationHelper.GetGuid(cd["EventUserGUID"], Guid.Empty));
                info.SetValue("AttendeeLastModified", DateTime.Now);
            }
        }
        catch (Exception e)
        {
            EventLogProvider eProvider = new EventLogProvider();
            eProvider.LogEvent("E", DateTime.Now, "PrepareInternalObject", e.InnerException.ToString());
        }

        return (info != null) ? info.Generalized : null;
    }

    #endregion
}
